//This file is part of Furius Iso Mount. Copyright 2008 Dean Harris (marcus_furius@hotmail.com)
//
//    Furius Iso Mount is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Furius Iso Mount is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Furius Iso Mount.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.IO;
using FuriusIsoMount;
using System.Diagnostics;
using Gtk;
using System.Threading;

namespace FuriusIsoMount
{
	
	/// <summary>
	/// Mounts an Image
	/// </summary>
	public static class ImageAction
	{
				
		/// <summary>
	    /// Mounts an Image
	    /// </summary>
		/// <param name="imageFile">The filename.</param>
        /// <param name="treeStore">The TreeStore used to store the current mounts</param>
		public static void Mount(string imageFile, TreeStore treeStore, ComboBoxEntry comboBoxEntry, string[] history)
		{
			FileInfo file = new FileInfo(imageFile);
			//Replace any characters that could screw us up.
			string mountLocation = Globals.HomeDirectory + file.Name.Replace(' ', '_');
			mountLocation = mountLocation.Replace('.', '_');
			try
			{
				// If an image is already mounted this will throw an IOException
				if (Directory.Exists(mountLocation))
				{
					Directory.Delete(mountLocation);
				}
				Directory.CreateDirectory(mountLocation);
				// Mount the image
		
				Process.Start("fuseiso '" + imageFile + "' '"+ mountLocation +"'");

				
				//Write an entry with instructions on how to remove manually if needed
				Log.Write(Globals.MountLog, 
				          string.Format(LocalizableResources.LogEntryMountSuccess, imageFile, mountLocation));
				
				// Add mount point and image to treeStore
				treeStore.AppendValues (mountLocation, imageFile);
				
				// Add the image to the history list
				AddItemToHistory(imageFile, comboBoxEntry, history);

			}
			catch (UnauthorizedAccessException exception)
			{
				Log.Write(Globals.MountLog, exception.Message);
				throw;
			}
			catch (IOException exception)
			{
				Log.Write(Globals.MountLog, exception.Message);
				throw;
			}
		}

		/// <summary>
		/// Add the image to the history list
		/// </summary>
		/// <param name="imageFile">A <see cref="System.String"/></param>
		/// <param name="comboBoxEntry">A <see cref="ComboBoxEntry"/></param>
		/// <param name="history">A <see cref="StringCollection"/></param>
		private static void AddItemToHistory(string imageFile, ComboBoxEntry comboBoxEntry, string[] history)
		{
			// Clear the comboBox			
			while(comboBoxEntry.Model.IterNChildren() != 0)
			{
				comboBoxEntry.RemoveText(0);
			}
						
			// If the image already exists in the list, remove it. We'll be adding it
			// to the top of the list.
			for (int i = 0; i < history.Length; i ++)
			{
				if (history[i] == imageFile)
				{
					Globals.RemoveFromArray(history, i);
				}
			}
			
			// Add the image to the top
			Globals.InsertIntoArray(history, imageFile, 0);
								
			// Finally rebuild the menu items and create our history
			using (StreamWriter streamWriter = new StreamWriter(Globals.HistoryList, false))
			{
				foreach(string historyItem in history)
				{
					if (historyItem != null)
					{
						comboBoxEntry.AppendText(historyItem);
						try
						{
							streamWriter.WriteLine(historyItem);
						}
						catch (IOException exception)
						{
							Log.Write(Globals.MountLog, exception.Message);
							throw;
						}
					}
				}
			}
			
		}
		
		/// <summary>
	    /// Unmounts an Image
	    /// </summary>
		/// <param name="mountPoint">The mount point.</param>
        /// <param name="treeStore">The TreeStore used to store the current mounts</param>
		/// <param name="iter">The TreeIter to remove from the TreeStore</param>
		public static void Unmount(string mountPoint, TreeStore treeStore, TreeIter iter, bool isShutDown)
		{
			try
			{
				// If the directory doesn't exist, it may have been
				// unmounted externally so just remove the TreeIter
				// from the TreeStore
				if (Directory.Exists(mountPoint))
				{
					// Start as a named process so we can ensure it
					// has completed before trying to remove the directory
					Process process = Process.Start("fusermount -u '"+ mountPoint +"'");			
					while (!process.HasExited)
			        {
			            Thread.Sleep(500); // sleep  			
			        }
					Thread.Sleep(500); // Patience my pretty!
					// When unmounting has completed delete the directory
					Directory.Delete(mountPoint);
					Log.Write(Globals.MountLog, 
				          string.Format(LocalizableResources.LogEntryUnmountSuccess, mountPoint));
				}
				if (!isShutDown)
				{
					// Remove mount point info from the TreeStore
					treeStore.Remove(ref iter);
				}
			}
			catch (UnauthorizedAccessException exception)
			{
				Log.Write(Globals.MountLog, exception.Message);
				throw;
			}
			catch (IOException exception)
			{
				Log.Write(Globals.MountLog, exception.Message);
				throw;
			}
		}
		
		/// <summary>
	    /// Opens Nautilus at the selected mount point
	    /// </summary>
		/// <param name="mountPoint">The mount point.</param>
		public static void Browse(string mountPoint)
		{
			 Process.Start("nautilus --browser '"+ mountPoint +"'");
			 Log.Write(Globals.MountLog, 
				          string.Format("{0}: nautilus --browser '{1}'", 
			                        LocalizableResources.LogEntryCommandAttempted, mountPoint));
		}
		
		/// <summary>
	    /// Starts the Nautilus CD burning wizard
	    /// </summary>
		/// <param name="imageFile">The Image file to burn</param>
		public static void Burn(string imageFile)
		{
			Process.Start("nautilus-cd-burner --source-iso='"+ imageFile +"'");
			Log.Write(Globals.MountLog, 
				          string.Format("{0}: nautilus-cd-burner --source-iso='{1}'", 
			                        LocalizableResources.LogEntryCommandAttempted, imageFile));
		}
		
	}
}
